Stan Math Library: stan/math/prim/scal/prob/gamma_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

gamma_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_GAMMA_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_GAMMA_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
5 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
6 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
7 #include <stan/math/prim/scal/err/check_less_or_equal.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/gamma_p.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/meta/length.hpp>
17 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
21 #include <stan/math/prim/scal/meta/return_type.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
24 #include <boost/random/gamma_distribution.hpp>
25 #include <boost/random/variate_generator.hpp>
26 #include <cmath>
27 #include <limits>
28
29 namespace stan {
30
31 namespace math {
32
33 template <typename T_y, typename T_shape, typename T_inv_scale>
34 typename return_type<T_y, T_shape, T_inv_scale>::type
35 gamma_ccdf_log(const T_y& y, const T_shape& alpha,
36 const T_inv_scale& beta) {
37 // Size checks
38 if (!(stan::length(y) && stan::length(alpha) && stan::length(beta)))
39 return 0.0;
40
41 typedef typename stan::partials_return_type<T_y, T_shape,
42 T_inv_scale>::type
43 T_partials_return;
44
45 // Error checks
46 static const char* function("stan::math::gamma_ccdf_log");
47
48 using stan::math::check_positive_finite;
49 using stan::math::check_not_nan;
50 using stan::math::check_consistent_sizes;
51 using stan::math::check_greater_or_equal;
52 using stan::math::check_less_or_equal;
53 using stan::math::check_nonnegative;
54 using stan::math::value_of;
55 using boost::math::tools::promote_args;
56 using std::exp;
57
58 T_partials_return P(0.0);
59
60 check_positive_finite(function, "Shape parameter", alpha);
61 check_positive_finite(function, "Scale parameter", beta);
62 check_not_nan(function, "Random variable", y);
63 check_nonnegative(function, "Random variable", y);
64 check_consistent_sizes(function,
65 "Random variable", y,
66 "Shape parameter", alpha,
67 "Scale Parameter", beta);
68
69 // Wrap arguments in vectors
70 VectorView<const T_y> y_vec(y);
71 VectorView<const T_shape> alpha_vec(alpha);
72 VectorView<const T_inv_scale> beta_vec(beta);
73 size_t N = max_size(y, alpha, beta);
74
75 OperandsAndPartials<T_y, T_shape, T_inv_scale>
76 operands_and_partials(y, alpha, beta);
77
78 // Explicit return for extreme values
79 // The gradients are technically ill-defined, but treated as zero
80
81 for (size_t i = 0; i < stan::length(y); i++) {
82 if (value_of(y_vec[i]) == 0)
83 return operands_and_partials.value(0.0);
84 }
85
86 // Compute ccdf_log and its gradients
87 using stan::math::gamma_p;
88 using stan::math::digamma;
89 using boost::math::tgamma;
90 using std::exp;
91 using std::pow;
92 using std::log;
93
94 // Cache a few expensive function calls if nu is a parameter
95 VectorBuilder<!is_constant_struct<T_shape>::value,
96 T_partials_return, T_shape> gamma_vec(stan::length(alpha));
97 VectorBuilder<!is_constant_struct<T_shape>::value,
98 T_partials_return, T_shape>
99 digamma_vec(stan::length(alpha));
100
101 if (!is_constant_struct<T_shape>::value) {
102 for (size_t i = 0; i < stan::length(alpha); i++) {
103 const T_partials_return alpha_dbl = value_of(alpha_vec[i]);
104 gamma_vec[i] = tgamma(alpha_dbl);
105 digamma_vec[i] = digamma(alpha_dbl);
106 }
107 }
108
109 // Compute vectorized ccdf_log and gradient
110 for (size_t n = 0; n < N; n++) {
111 // Explicit results for extreme values
112 // The gradients are technically ill-defined, but treated as zero
113 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity())
114 return operands_and_partials.value(stan::math::negative_infinity());
115
116 // Pull out values
117 const T_partials_return y_dbl = value_of(y_vec[n]);
118 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
119 const T_partials_return beta_dbl = value_of(beta_vec[n]);
120
121 // Compute
122 const T_partials_return Pn = 1.0 - gamma_p(alpha_dbl, beta_dbl * y_dbl);
123
124 P += log(Pn);
125
126 if (!is_constant_struct<T_y>::value)
127 operands_and_partials.d_x1[n] -= beta_dbl * exp(-beta_dbl * y_dbl)
128 * pow(beta_dbl * y_dbl, alpha_dbl-1) / tgamma(alpha_dbl) / Pn;
129 if (!is_constant_struct<T_shape>::value)
130 operands_and_partials.d_x2[n]
131 += stan::math::grad_reg_inc_gamma(alpha_dbl, beta_dbl
132 * y_dbl, gamma_vec[n],
133 digamma_vec[n]) / Pn;
134 if (!is_constant_struct<T_inv_scale>::value)
135 operands_and_partials.d_x3[n] -= y_dbl * exp(-beta_dbl * y_dbl)
136 * pow(beta_dbl * y_dbl, alpha_dbl-1) / tgamma(alpha_dbl) / Pn;
137 }
138
139 return operands_and_partials.value(P);
140 }
141 }
142 }
143
144 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::grad_reg_inc_gamma
T grad_reg_inc_gamma(T a, T z, T g, T dig, T precision=1e-6)
Definition: grad_reg_inc_gamma.hpp:16
return_type.hpp
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
check_less_or_equal.hpp
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::gamma_ccdf_log
return_type< T_y, T_shape, T_inv_scale >::type gamma_ccdf_log(const T_y &y, const T_shape &alpha, const T_inv_scale &beta)
Definition: gamma_ccdf_log.hpp:35
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_less_or_equal
bool check_less_or_equal(const char *function, const char *name, const T_y &y, const T_high &high)
Return true if y is less or equal to high.
Definition: check_less_or_equal.hpp:81
length.hpp
stan::math::gamma_p
fvar< T > gamma_p(const fvar< T > &x1, const fvar< T > &x2)
Definition: gamma_p.hpp:15
gamma_p.hpp
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::math::tgamma
fvar< T > tgamma(const fvar< T > &x)
Definition: tgamma.hpp:15
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.